#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    // Sync video refresh rate
    ofSetVerticalSync(true);
    // Set background color
    ofBackground(170, 170, 170);
    ofEnableSmoothing();
    ofHideCursor();
    
    bgm.loadSound("bgm.mp3");
    bgm.setLoop(true);
    bgm.play();
    bgm.setVolume(0.2);
    
    button.loadSound("button.wav");
    
    playGame = false;
    firstGame = true;
    won = false;
    
    font.loadFont("font.ttf", 15, true, true);
    timeFont.loadFont("time.ttf", 25, true, true);
    
    background.loadImage("background.png");
    cursor.loadImage("cursor.png");
    background.resize(ofGetWindowWidth(), ofGetWindowHeight());
    
    minute = 5;
    second = 0;

    
}

//--------------------------------------------------------------
void ofApp::update(){
    if (playGame) {
        if (!p.isAlive()) playGame = false;
        int time = ofGetElapsedTimeMillis();
        
        if (time > 1000) {
            if (second == 0) {
                if (minute == 0) {
                    playGame = false;
                    won = true;
                } else {
                    --minute;
                    second = 59;
                }
            } else {
                --second;
            }
            ofResetElapsedTimeCounter();
        }
        
        for (int i = 0; i < monsters.size(); ++i) {
            if (!monsters[i]->update()) {
                p.damage(monsters[i]->damageAmount());
            }
        }
        
        // update time
        ++tick;
        if (tick > 200) {
            createMonsters();
            tick = 0;
            ++timeAfterGame;
        }
        
        ++delayTick;
        if (delayTick > 5) {
            checkDefense();
            delayTick = 0;
        }
        
        // when monsters reach end, start attacking castle
        for (int i = 0; i < monsters.size(); ++i) {
            if (monsters[i]->getLocation() + monsters[i]->getWidth() == p.playerLocation()) {
                monsters[i]->attack();
            }
        }
        
        if (useNuclear && !alreadyUsedNuclear) {
            numKills += monsters.size();
            if (numKills > KILL_LIMIT) numKills = KILL_LIMIT;
            for (int i = 0; i < monsters.size(); ++i) {
                effects.push_back(new KillEffect(monsters[i]->getLocation() + monsters[i]->getWidth()/2, monsters[i]->getYLocation() + monsters[i]->getHeight()/2));
            }
            monsters.clear();
            selected = -1;
            alreadyUsedNuclear = true;
        }
    } else {
        if (firstGame) {
            if (keyDown[13]) {
                button.play();
                playGame = true;
                firstGame = false;
                ofResetElapsedTimeCounter();
                numKills = 0;
                useNuclear = false;
                alreadyUsedNuclear = false;
            }
        } else {
            if (keyDown['y']) {
                button.play();
                playGame = true;
                monsters.clear();
                p = Player();
                timeAfterGame = 0;
                second = 0;
                minute = 5;
                numKills = 0;
                ofResetElapsedTimeCounter();
                useNuclear = false;
                alreadyUsedNuclear = false;
            } else if (keyDown['n']) {
                std::exit(0);
            }
        }

    }

}


//--------------------------------------------------------------
void ofApp::draw(){
    ofSetColor(255, 255, 255, 200);
    background.draw(0, 0);
    if (playGame){
        
        ofSetColor(0, 0, 0);
        ofNoFill();
        ofRect(ofGetWindowWidth()/2 + 100, 30, KILL_LIMIT * 10, 30);
        if (numKills == KILL_LIMIT) {
            ofSetColor(0, 0, 255);
        } else {
            ofSetColor(211, 211, 211, 150);
        }
        ofFill();
        ofRect(ofGetWindowWidth()/2 + 100, 30, numKills * 10, 30);
        
        p.draw();
        for (int i = monsters.size() - 1; i >= 0; --i) monsters[i]->draw();
        for (int i = effects.size() - 1; i >= 0; --i) {
            effects[i]->decreaseDuration();
            bool clearAll = effects[i]->draw();
            if (effects[i]->isDone()) {
                if (effects[i]->type() == 3) {
                    useNuclear = false;
                    alreadyUsedNuclear = false;
                }
                effects.erase(effects.begin() + i);
            }
            
            if (effects[i]->type() == 3) { // nuclear
                if (clearAll) {
                    useNuclear = true;
                }
            }
        }
        
        ofSetColor(211, 211, 211, 150);
        string secondString = ofToString(second);
        if (secondString.length() == 1) secondString = "0" + secondString;
        string time = ofToString(minute) + ":" + secondString;
        timeFont.drawString(time, ofGetWindowWidth()/2 - 50, 50);
        
    } else {
        ofSetColor(57, 255, 20, 150);
        if (firstGame) {
            string s = "Press enter to begin\n 1 is \"C\" 2 is \"D flat\" and so on...\n Click on the monster to hear the note(s)\n When the kill gage is full, press n to use nuclear bomb. \n See if you can stand 5 minutes!";
            font.drawString(s, (ofGetWindowWidth() - font.stringWidth(s))/2, (ofGetWindowHeight() - font.stringHeight(s))/2);
        } else if (won) {
            string s = "Congrats! You won!\n Play again? (y/n)";
            font.drawString(s, (ofGetWindowWidth() - font.stringWidth(s))/2, (ofGetWindowHeight() - font.stringHeight(s))/2);
        } else {
            string s = "Sorry. You lost. Boo...\n Play again? (y/n)";
            font.drawString(s, (ofGetWindowWidth() - font.stringWidth(s))/2, (ofGetWindowHeight() - font.stringHeight(s))/2);
        }
    }
    
    ofSetColor(255, 255, 255, 255);
    cursor.draw(mouseX, mouseY);
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    keyDown[key] = true;
    
    if (key == 'n') {
        if (numKills >= KILL_LIMIT) {
            effects.push_back(new NuclearEffect());
            numKills = 0;
        }
    }
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){
    keyDown[key] = false;
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
    mouseX = x;
    mouseY = y;
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
    for (int i = 0; i < monsters.size(); ++i) {
        if (monsters[i]->isMonster(x, y)) {
            monsters[i]->select();
            monsters[i]->play();
            if (selected >= 0 && selected != i) monsters[selected]->unselect();
            selected = i;
            break;
        }
    }
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){
}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){
    background.resize(w, h);
}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){
}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){
}

void ofApp::createMonsters() {
    if (ofRandom(1.0) < 0.6) {
        vector<Note> notes;
        notes.push_back(Notes(int(ofRandom(12))));
        monsters.push_back(new SingleNoteMonster(notes));
    }
    if (timeAfterGame > 15) {
        if (ofRandom(1.0) < 0.7) {
            int one = int(ofRandom(11));
            int two = int(ofRandom(one + 1, 12));
            vector<Note> notes;
            notes.push_back(Notes(one));
            notes.push_back(Notes(two));
            monsters.push_back(new DoubleNoteMonster(notes));
            return;
        }
    }
    if (timeAfterGame > 30) {
        if (ofRandom(1.0) < 0.7) {
            vector<Note> notes;
            int one = int(ofRandom(10));
            int two = int(ofRandom(one, 11));
            int three = int(ofRandom(two, 12));
            notes.push_back(Notes(one));
            notes.push_back(Notes(two));
            notes.push_back(Notes(three));
            monsters.push_back(new ThreeNoteMonster(notes));
            return;
        }
    }
    
    if (timeAfterGame > 45) {
        if (ofRandom(1.0) < 0.7) {
            vector<Note> notes;
            int one = int(ofRandom(9));
            int two = int(ofRandom(one, 10));
            int three = int(ofRandom(two, 11));
            int four = int(ofRandom(three, 12));
            notes.push_back(Notes(one));
            notes.push_back(Notes(two));
            notes.push_back(Notes(three));
            notes.push_back(Notes(four));
            monsters.push_back(new FourNoteMonster(notes));
        }
    }
}

void ofApp::checkDefense() {
   
    vector<Note> guess;
    if (keyDown['1']) {
        guess.push_back(Note(Notes::C));
    }
    if (keyDown['2']) {
        guess.push_back(Note(Notes::Db));
    }
    if (keyDown['3']) {
        guess.push_back(Note(Notes::D));
    }
    if (keyDown['4']) {
        guess.push_back(Note(Notes::Eb));
    }
    if (keyDown['5']) {
        guess.push_back(Note(Notes::E));
    }
    if (keyDown['6']) {
        guess.push_back(Note(Notes::F));
    }
    if (keyDown['7']) {
        guess.push_back(Note(Notes::Gb));
    }
    if (keyDown['8']) {
        guess.push_back(Note(Notes::G));
    }
    if (keyDown['9']) {
        guess.push_back(Note(Notes::Ab));
    }
    if (keyDown['0']) {
        guess.push_back(Note(Notes::A));
    }
    if (keyDown['-']) {
        guess.push_back(Note(Notes::Bb));
    }
    if (keyDown['=']) {
        guess.push_back(Note(Notes::B));
    }
    
    if (guess.empty()) return;  // no guess
    
    for (int i = 0; i < guess.size(); ++i) guess[i].play();
    
    if (selected < 0) return;  // no monster selected
    
    if (monsters[selected]->checkAnswer(guess)){
        
        // add effects for killing
        effects.push_back(new KillEffect(monsters[selected]->getLocation() + monsters[selected]->getWidth()/2, monsters[selected]->getYLocation() + monsters[selected]->getHeight()/2));
        
        if (numKills < KILL_LIMIT) ++numKills;
        monsters.erase(monsters.begin() + selected);
        selected = -1;
    } else {
        // add effects for attack
        effects.push_back(new AttackEffect(monsters[selected]->getLocation(), monsters[selected]->getYLocation(), monsters[selected]->getWidth(), monsters[selected]->getHeight()));
    }
}